@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
  *
@@ -21,43 +20,32 @@
 #ifndef __LIBTRACKER_COMMON_H__
 #define __LIBTRACKER_COMMON_H__
 
-#include <glib.h>
-
-G_BEGIN_DECLS
-
-#if !defined (TRACKER_ENABLE_INTERNALS) && !defined (TRACKER_COMPILATION)
-#error "TRACKER_ENABLE_INTERNALS not defined, this must be defined to use tracker's internal functions"
+#if !defined (__LIBTRACKER_COMMON_INSIDE__) && !defined (TRACKER_COMPILATION)
+#error "only <libtracker-common/tracker-common.h> must be included directly."
 #endif
 
+#include <glib.h>
+
 #define __LIBTRACKER_COMMON_INSIDE__
 
-/* Not installed headers */
-#if 0
-#include "tracker-dbus.h"
-#include "tracker-ioprio.h"
-#include "tracker-log.h"
-#include "tracker-os-dependant.h"
-#include "tracker-power.h"
-#include "tracker-storage.h"
-#endif
-
-/* Installed headers */
 #include "tracker-class.h"
 #include "tracker-config-file.h"
+#include "tracker-date-time.h"
+#include "tracker-dbus.h"
 #include "tracker-file-utils.h"
+#include "tracker-ioprio.h"
 #include "tracker-keyfile-object.h"
 #include "tracker-language.h"
+#include "tracker-log.h"
 #include "tracker-namespace.h"
 #include "tracker-ontology.h"
+#include "tracker-ontologies.h"
+#include "tracker-os-dependant.h"
+#include "tracker-power.h"
 #include "tracker-property.h"
-#include "tracker-sparql-builder.h"
-#include "tracker-statement-list.h"
 #include "tracker-type-utils.h"
 #include "tracker-utils.h"
 
 #undef __LIBTRACKER_COMMON_INSIDE__
 
-
-G_END_DECLS
-
 #endif /* __LIBTRACKER_COMMON_H__ */
