@@ -46,11 +46,9 @@ tracker_data_query_rdf_type (gint id)
 	iface = tracker_db_manager_get_db_interface ();
 
 	stmt = tracker_db_interface_create_statement (iface,
-	                                              "SELECT \"rdfs:Resource\".\"Uri\" "
+	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"rdf:type\") "
 	                                              "FROM \"rdfs:Resource_rdf:type\" "
-	                                              "INNER JOIN \"rdfs:Resource\" "
-	                                              "ON \"rdfs:Resource_rdf:type\".\"rdf:type\" = \"rdfs:Resource\".\"ID\" "
-	                                              "WHERE \"rdfs:Resource_rdf:type\".\"ID\" = ?");
+	                                              "WHERE ID = ?");
 
 	tracker_db_statement_bind_int (stmt, 0, id);
 	cursor = tracker_db_statement_start_cursor (stmt, NULL);
@@ -64,7 +62,16 @@ tracker_data_query_rdf_type (gint id)
 
 		ret = g_ptr_array_sized_new (20);
 		while (tracker_db_cursor_iter_next (cursor)) {
-			g_ptr_array_add (ret, g_strdup (tracker_db_cursor_get_string (cursor, 0)));
+			const gchar *class_uri;
+			TrackerClass *cl;
+
+			class_uri = tracker_db_cursor_get_string (cursor, 0);
+			cl = tracker_ontologies_get_class_by_uri (class_uri);
+			if (!cl) {
+				g_critical ("Unknown class %s", class_uri);
+				continue;
+			}
+			g_ptr_array_add (ret, cl);
 		}
 		g_object_unref (cursor);
 	}
@@ -85,7 +92,7 @@ tracker_data_query_resource_id (const gchar *uri)
 	iface = tracker_db_manager_get_db_interface ();
 
 	stmt = tracker_db_interface_create_statement (iface,
-	                                              "SELECT ID FROM \"rdfs:Resource\" WHERE Uri = ?");
+	                                              "SELECT ID FROM Resource WHERE Uri = ?");
 	tracker_db_statement_bind_text (stmt, 0, uri);
 	cursor = tracker_db_statement_start_cursor (stmt, NULL);
 	g_object_unref (stmt);
