@@ -43,10 +43,9 @@
 #include <enca.h>
 #endif
 
-#include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontology.h>
-#include <libtracker-common/tracker-utils.h>
-#include <libtracker-common/tracker-type-utils.h>
+#include <libtracker-common/tracker-common.h>
+
+#include <libtracker-client/tracker.h>
 
 #include <libtracker-extract/tracker-extract.h>
 
@@ -91,6 +90,7 @@ typedef struct {
 	guint32 length;
 	gchar *performer1;
 	gchar *performer2;
+	gchar *composer;
 	gchar *publisher;
 	gchar *recording_time;
 	gchar *release_time;
@@ -139,7 +139,8 @@ typedef enum {
 	ID3V24_TPUB,
 	ID3V24_TRCK,
 	ID3V24_TYER,
-	ID3V24_TOLY
+	ID3V24_TOLY,
+	ID3V24_TCOM,
 } id3v24frame;
 
 typedef struct {
@@ -161,7 +162,8 @@ typedef struct {
 	gchar *copyright;
 	gchar *publisher;
 	gchar *comment;
-
+	gchar *composer;
+	gchar *composer_uri;
 	gint track_number;
 	gint track_count;
 
@@ -201,6 +203,7 @@ static const struct {
 	{ "APIC", ID3V24_APIC },
 	{ "COMM", ID3V24_COMM },
 	{ "TALB", ID3V24_TALB },
+	{ "TCOM", ID3V24_TCOM },
 	{ "TCON", ID3V24_TCON },
 	{ "TCOP", ID3V24_TCOP },
 	{ "TDRC", ID3V24_TDRC },
@@ -229,9 +232,9 @@ static const struct {
 	{ "TCO", ID3V2_TCO },
 	{ "TCR", ID3V2_TCR },
 	{ "TLE", ID3V2_TLE },
-	{ "TPB", ID3V2_TPB },
 	{ "TP1", ID3V2_TP1 },
 	{ "TP2", ID3V2_TP2 },
+	{ "TPB", ID3V2_TPB },
 	{ "TT1", ID3V2_TT1 },
 	{ "TT2", ID3V2_TT2 },
 	{ "TT3", ID3V2_TT3 },
@@ -741,7 +744,7 @@ get_id3 (const gchar *data,
 	pos += 30;
 	year = g_convert (pos, 4, "UTF-8", encoding, NULL, NULL, NULL);
 	if (atoi (year) > 0) {
-		id3->recording_time = tracker_date_format (year);
+		id3->recording_time = tracker_date_guess (year);
 	}
 	g_free (year);
 
@@ -1234,11 +1237,12 @@ get_id3v24_tags (const gchar          *data,
 				tag->copyright = word;
 				break;
 			case ID3V24_TDRC:
-				tag->recording_time = tracker_date_format (word);
+				tag->recording_time = tracker_date_guess (word);
 				g_free (word);
 				break;
 			case ID3V24_TDRL:
-				tag->release_time = word;
+				tag->release_time = tracker_date_guess (word);
+				g_free (word);
 				break;
 			case ID3V24_TEXT:
 				tag->text = word;
@@ -1246,6 +1250,9 @@ get_id3v24_tags (const gchar          *data,
 			case ID3V24_TOLY:
 				tag->toly = word;
 				break;
+			case ID3V24_TCOM:
+				tag->composer = word;
+				break;
 			case ID3V24_TIT1:
 				tag->title1 = word;
 				break;
@@ -1285,7 +1292,7 @@ get_id3v24_tags (const gchar          *data,
 			}
 			case ID3V24_TYER:
 				if (atoi (word) > 0) {
-					tag->recording_time = tracker_date_format (word);
+					tag->recording_time = tracker_date_guess (word);
 				}
 				g_free (word);
 				break;
@@ -1487,7 +1494,7 @@ get_id3v23_tags (const gchar          *data,
 			}
 			case ID3V24_TYER:
 				if (atoi (word) > 0) {
-					tag->recording_time = tracker_date_format (word);
+					tag->recording_time = tracker_date_guess (word);
 				}
 				g_free (word);
 				break;
@@ -1622,7 +1629,7 @@ get_id3v20_tags (const gchar          *data,
 				break;
 			case ID3V2_TYE:
 				if (atoi (word) > 0) {
-					tag->recording_time = tracker_date_format (word);
+					tag->recording_time = tracker_date_guess (word);
 				}
 				g_free (word);
 				break;
@@ -1941,6 +1948,11 @@ extract_mp3 (const gchar          *uri,
 	md.title = tracker_coalesce (4, md.id3v24.title2, md.id3v23.title2, md.id3v22.title2, md.id3v1.title);
 	md.lyricist = tracker_coalesce (2, md.id3v24.text, md.id3v23.toly);
 
+	md.composer = tracker_coalesce (3,
+	                                md.id3v24.composer,
+	                                md.id3v23.composer,
+	                                md.id3v22.composer);
+
 	md.performer = tracker_coalesce (7,
 	                                 md.id3v24.performer1, md.id3v24.performer2,
 	                                 md.id3v23.performer1, md.id3v23.performer2,
@@ -1996,8 +2008,21 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_object_unvalidated (preupdate, md.performer);
 
 		tracker_sparql_builder_insert_close (preupdate);
+		/* do not delete artist, needed by albumart */
+	}
 
-		g_free (md.performer);
+	if (md.composer) {
+		md.composer_uri = tracker_uri_printf_escaped ("urn:artist:%s", md.composer);
+
+		tracker_sparql_builder_insert_open (preupdate, NULL);
+
+		tracker_sparql_builder_subject_iri (preupdate, md.composer_uri);
+		tracker_sparql_builder_predicate (preupdate, "a");
+		tracker_sparql_builder_object (preupdate, "nmm:Artist");
+		tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
+		tracker_sparql_builder_object_unvalidated (preupdate, md.composer);
+
+		tracker_sparql_builder_insert_close (preupdate);
 	}
 
 	if (md.lyricist) {
@@ -2071,6 +2096,12 @@ extract_mp3 (const gchar          *uri,
 		g_free (md.performer_uri);
 	}
 
+	if (md.composer_uri) {
+		tracker_sparql_builder_predicate (metadata, "nmm:composer");
+		tracker_sparql_builder_object_iri (metadata, md.composer_uri);
+		g_free (md.composer_uri);
+	}
+
 	if (md.album_uri) {
 		tracker_sparql_builder_predicate (metadata, "nmm:musicAlbum");
 		tracker_sparql_builder_object_iri (metadata, md.album_uri);
@@ -2123,10 +2154,12 @@ extract_mp3 (const gchar          *uri,
 	tracker_albumart_process (md.albumart_data,
 	                          md.albumart_size,
 	                          md.albumart_mime,
-	                          NULL,
+	                          md.performer,
 	                          md.album,
 	                          filename);
 
+	g_free (md.performer);
+	g_free (md.composer);
 	g_free (md.album);
 	g_free (md.title);
 	g_free (md.albumart_data);
