@@ -22,14 +22,14 @@
 
 #include "config.h"
 
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 
 #include "tracker-events.h"
 
 
 typedef struct {
 	GHashTable *allowances;
-	GPtrArray *events;
+	GArray *events;
 } EventsPrivate;
 
 static GStaticPrivate private_key = G_STATIC_PRIVATE_INIT;
@@ -38,60 +38,46 @@ static void
 tracker_events_add_allow (const gchar *rdf_class)
 {
 	EventsPrivate *private;
+	TrackerClass *cl;
 
 	private = g_static_private_get (&private_key);
 	g_return_if_fail (private != NULL);
 
-	g_hash_table_insert (private->allowances, g_strdup (rdf_class),
+	cl = tracker_ontologies_get_class_by_uri (rdf_class);
+	if (!cl) {
+		g_critical ("Unknown class %s", rdf_class);
+		return;
+	}
+
+	g_hash_table_insert (private->allowances, cl,
 	                     GINT_TO_POINTER (TRUE));
 }
 
 static gboolean
-is_allowed (EventsPrivate *private, const gchar *rdf_class)
+is_allowed (EventsPrivate *private, TrackerClass *rdf_class)
 {
 	return (g_hash_table_lookup (private->allowances, rdf_class) != NULL) ? TRUE : FALSE;
 }
 
 static void
 prepare_event_for_rdf_type (EventsPrivate *private,
-                            const gchar *rdf_class ,
+                            TrackerClass  *rdf_class ,
                             const gchar *uri,
                             TrackerDBusEventsType type,
                             const gchar *predicate)
 {
-	GValueArray *event;
-	GValue uri_value = { 0 , };
-	GValue rdfclass_value = { 0 , };
-	GValue type_value = { 0 , };
-	GValue predicate_value = { 0 , };
+	TrackerEvent event;
 
 	if (!private->events) {
-		private->events = g_ptr_array_new ();
+		private->events = g_array_new (TRUE, FALSE, sizeof (TrackerEvent));
 	}
 
-	g_value_init (&uri_value, G_TYPE_STRING);
-	g_value_init (&predicate_value, G_TYPE_STRING);
-	g_value_init (&rdfclass_value, G_TYPE_STRING);
-	g_value_init (&type_value, G_TYPE_INT);
+	event.type = type;
+	event.class = rdf_class;
+	event.predicate = tracker_ontologies_get_property_by_uri (predicate);
+	event.subject = g_strdup (uri);
 
-	event = g_value_array_new (4);
-
-	g_value_set_string (&uri_value, uri);
-	g_value_set_string (&predicate_value, predicate);
-	g_value_set_string (&rdfclass_value, rdf_class);
-	g_value_set_int (&type_value, type);
-
-	g_value_array_append (event, &uri_value);
-	g_value_array_append (event, &predicate_value);
-	g_value_array_append (event, &rdfclass_value);
-	g_value_array_append (event, &type_value);
-
-	g_ptr_array_add (private->events, event);
-
-	g_value_unset (&uri_value);
-	g_value_unset (&rdfclass_value);
-	g_value_unset (&type_value);
-	g_value_unset (&predicate_value);
+	g_array_append_val (private->events, event);
 }
 
 void
@@ -103,6 +89,7 @@ tracker_events_insert (const gchar *uri,
 {
 	EventsPrivate *private;
 
+	g_return_if_fail (rdf_types || type != TRACKER_DBUS_EVENTS_TYPE_UPDATE);
 	private = g_static_private_get (&private_key);
 	g_return_if_fail (private != NULL);
 
@@ -117,28 +104,24 @@ tracker_events_insert (const gchar *uri,
 
 				prepare_event_for_rdf_type (private, rdf_types->pdata[i],
 				                            uri, type, predicate);
-
-				/* Only once match is needed */
-				break;
 			}
 		}
-	} else if (type == TRACKER_DBUS_EVENTS_TYPE_UPDATE) {
-		/* In this case we had an INSERT for a resource that didn't exist
-		 * yet, but it was not the rdf:type predicate being inserted */
-		if (is_allowed (private, (gpointer) TRACKER_RDFS_PREFIX "Resource")) {
-			prepare_event_for_rdf_type (private,
-			                            (gpointer) TRACKER_RDFS_PREFIX "Resource",
-			                            uri, type, predicate);
-		}
 	} else {
+		TrackerClass *class = tracker_ontologies_get_class_by_uri (object);
 		/* In case of delete and create, object is the rdf:type */
-		if (is_allowed (private, (gpointer) object)) {
-			prepare_event_for_rdf_type (private, (gpointer) object,
+		if (is_allowed (private, class)) {
+			prepare_event_for_rdf_type (private, class,
 			                            uri, type, predicate);
 		}
 	}
 }
 
+static void
+tracker_event_destroy (TrackerEvent *event)
+{
+	g_free (event->subject);
+}
+
 void
 tracker_events_reset (void)
 {
@@ -150,15 +133,15 @@ tracker_events_reset (void)
 
 	if (private->events) {
 		for (i = 0; i < private->events->len; i++) {
-			g_value_array_free (private->events->pdata[i]);
+			tracker_event_destroy (&g_array_index (private->events, TrackerEvent, i));
 		}
-		g_ptr_array_free (private->events, TRUE);
+		g_array_free (private->events, TRUE);
 
 		private->events = NULL;
 	}
 }
 
-GPtrArray *
+GArray *
 tracker_events_get_pending (void)
 {
 	EventsPrivate *private;
@@ -189,9 +172,7 @@ tracker_events_init (TrackerNotifyClassGetter callback)
 	                      private,
 	                      (GDestroyNotify) free_private);
 
-	private->allowances = g_hash_table_new_full (g_str_hash, g_str_equal,
-	                                             (GDestroyNotify) g_free,
-	                                             (GDestroyNotify) NULL);
+	private->allowances = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	private->events = NULL;
 
